package ch.polgrabia.demos.client;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.AsyncResult;
import io.vertx.core.http.HttpClient;
import io.vertx.core.http.HttpClientOptions;
import io.vertx.core.http.WebSocket;
import io.vertx.core.http.impl.WebSocketClientImpl;
import io.vertx.core.impl.CloseFuture;
import io.vertx.core.impl.VertxInternal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.CompletableFuture;

public class WebsocketClientParticle extends AbstractVerticle {
    private static final Logger logger = LoggerFactory.getLogger(WebsocketClientParticle.class);
    private final String hostname;
    private final int port;
    // path maps to channel
    private final String path;
    private HttpClient client;
    private boolean isShuttingDownActivated = false;
    private final Object runningMonitor = new Object();

    public WebsocketClientParticle(String hostname, int port, String path) {
        this.hostname = hostname;
        this.port = port;
        this.path = path;
    }

    @Override
    public void start() {
        this.client = getVertx()
                .createHttpClient();
        new WebSocketClientImpl((VertxInternal) getVertx(), new HttpClientOptions(), new CloseFuture())
                .webSocket(port, hostname, path, this::handleMessage);
    }

    private void handleMessage(AsyncResult<WebSocket> webSocketAsyncResult) {
        logger.info("Handling websocket");
        if (!webSocketAsyncResult.succeeded()) {
            logger.warn("Websocket handler connection failed", webSocketAsyncResult.cause());
            try {
                stop();
            } catch (Exception e) {
                logger.error("Failed to stop particle", e);
            }
            return;
        }
        WebSocket webSocket = webSocketAsyncResult.result();
        vertx.setPeriodic(100, new ClientPeriodicStdinScanner(
                webSocket,
                System.in
        ));

        webSocket
                .binaryMessageHandler(msg -> logger.info("[{}] Got message: {}", path, msg))
                .closeHandler(unused -> {
                    try {
                        stop();
                    } catch (Exception e) {
                        logger.error("Got an error while stopping", e);
                    }
                });

    }

    @Override
    public void stop() {
        isShuttingDownActivated = true;
        client.close();
        synchronized (runningMonitor) {
            runningMonitor.notifyAll();
        }
    }

    public CompletableFuture<Boolean> waitingUntilFinished() {
        return CompletableFuture.supplyAsync(() -> {
            while (!isShuttingDownActivated) {
                synchronized (runningMonitor) {
                    try {
                        runningMonitor.wait();
                        return true;
                    } catch (InterruptedException e) {
                        logger.warn("Got interrupted", e);
                    }
                }
            }
            return false;
        });
    }

}
