package ch.polgrabia.demos.server;

import io.vertx.core.Handler;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.http.ServerWebSocket;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class WebsocketServerHandler implements Handler<ServerWebSocket> {
    private static final Logger logger = LoggerFactory.getLogger(WebsocketServerHandler.class);

    @Override
    public void handle(ServerWebSocket serverWebSocket) {
        serverWebSocket.writeBinaryMessage(Buffer.buffer("Hello"));
        serverWebSocket.handler(new WebsocketMessageHandler(serverWebSocket));
    }

    static class WebsocketMessageHandler implements Handler<Buffer> {

        private final ServerWebSocket serverWebSocket;

        public WebsocketMessageHandler(ServerWebSocket serverWebSocket) {
            this.serverWebSocket = serverWebSocket;
            this.serverWebSocket.setWriteQueueMaxSize(0);
        }

        @Override
        public void handle(Buffer s) {
            logger.info("[{}] Got message {}", serverWebSocket.path(), s);
            serverWebSocket.writeBinaryMessage(Buffer.buffer("Pong: " + s.toString() + "\n")); // TBD send to kafka
        }
    }

}
